#!/usr/bin/env node

const asar = require("asar");
const async = require("async");
const chalk = require("chalk");
const del = require("del");
const fs = require("fs");
const manifest = require("../package.json");
const path = require("path");
const program = require("commander");
const sass = require("node-sass");

const Enquirer = require("enquirer");

const enquirer = new Enquirer();

program
    .version(manifest.version, "-v, --version");

program
    .command("generate <type>")
    .description("Generate a new project. <type> must be \"extension\" or \"theme\".")
    .action(type => {
        // Generate a new theme..
        if (type === "theme" || type === "themes") {
            console.clear();
            console.log("Please answer the following questions:\r\n");
            const questions = [
                {
                    type: "text",
                    name: "name",
                    message: "What is the name of your theme?",
                    format(input = this.input) {
                        return input.replace(/[^a-z0-9_-]/gi, "").toLowerCase();
                    },
                    validate(value) {
                        if (value.trim() === "") return false;
                        return true;
                    }
                },
                {
                    type: "text",
                    name: "description",
                    message: "What is the description of your theme?"
                },
                {
                    type: "text",
                    name: "username",
                    message: "What is the author's username?"
                }
            ];
            enquirer.prompt(questions).then((answers) => {
                // User has answered all of our questions..
                if (Object.keys(answers).length >= questions.length) {
                    const name = answers.name.trim().toLowerCase();

                    // Create directory..
                    fs.mkdirSync(`${process.cwd()}/${name}`);

                    // Create files and directories from templates..
                    createTemplateContents(`${__dirname}/templates/themes`, name);
                    createManifest({ name, description: answers.description, author: answers.username, main: "scss/theme.scss" });

                    console.log(`\r\n${chalk.green("Done!")} Your theme was generated in "${chalk.green(process.cwd())}${chalk.green("\\")}${chalk.green(name)}". Have fun!`);
                } else {
                    console.log(`\r\n${chalk.green("Done!")} But you didn't answer all of our questions, so your theme wasn't generated.`);
                }
            }).catch(console.error);
        }
        // Generate a new extension..
        else if (type === "extension" || type === "extensions") {
            console.clear();
            console.log("Please answer the following questions:\r\n");
            const questions = [
                {
                    type: "text",
                    name: "name",
                    message: "What is the name of your extension?",
                    format(input = this.input) {
                        return input.replace(/[^a-z0-9_-]/gi, "").toLowerCase();
                    },
                    validate(value) {
                        if (value.trim() === "") return false;
                        return true;
                    }
                },
                {
                    type: "text",
                    name: "description",
                    message: "What is the description of your extension?"
                },
                {
                    type: "text",
                    name: "username",
                    message: "What is the author's username?"
                }
            ];
            enquirer.prompt(questions).then((answers) => {
                // User has answered all of our questions..
                if (Object.keys(answers).length >= questions.length) {
                    const name = answers.name.toLowerCase();

                    // Create directory..
                    fs.mkdirSync(`${process.cwd()}/${name}`);

                    // Create files and directories from templates..
                    createTemplateContents(`${__dirname}/templates/extensions`, name);
                    createManifest({ name, description: answers.description, author: answers.username, main: "index.js", menu: {}, settings: [] });
                    fs.writeFileSync(`${process.cwd()}/${name}/${name}.html`, "<p>This is your new extension!</p>", "utf8");

                    console.log(`\r\n${chalk.green("Done!")} Your extension was generated in "${chalk.green(process.cwd())}${chalk.green("\\")}${chalk.green(name)}". Have fun!`);
                } else {
                    console.log(`\r\n${chalk.green("Done!")} But you didn't answer all of our questions, so your extension wasn't generated.`);
                }
            }).catch(console.error);
        }
        // Invalid type..
        else {
            console.log("Depending on what you want to generate, please use \"onik generate theme\" or \"onik generate extension\".");
        }
    });

program
    .command("build")
    .description("Build the current extension or theme.")
    .action(() => {
        // Check if manifest exists..
        if (fs.existsSync(`${process.cwd()}/package.json`)) {
            let pjManifest = require(`${process.cwd()}/package.json`);

            // "name" of the project is mandatory..
            if (typeof pjManifest.name === "undefined") {
                return console.log(`\r\n${chalk.red("Error!")} Cannot find value for "name" in package.json.`);
            }

            // "main" is either a .scss or .js file..
            if (typeof pjManifest.main === "undefined") {
                return console.log(`\r\n${chalk.red("Error!")} Cannot find value for "main" in package.json.`);
            }

            // Build the theme..
            if (path.extname(pjManifest.main) === ".scss") {
                async.series({
                    theme(callback) {
                        console.log(`Compressing ${chalk.yellow(path.join(process.cwd(), pjManifest.main))}..`);
                        sass.render({
                            file: path.join(process.cwd(), pjManifest.main),
                            outputStyle: "expanded",
                            sourceMap: true,
                            sourceMapContents: true,
                            precision: 6,
                            outFile: path.join(process.cwd(), `temp/${pjManifest.name}.css`)
                        }, (err, {css}) => {
                            if (!err) {
                                fs.mkdir(path.join(process.cwd(), "temp"), { recursive: true }, (err) => {
                                    if (err) {
                                        callback(err);
                                        return console.log(`Error: ${chalk.red(err)}`);
                                    }
                                    fs.writeFile(path.join(process.cwd(), `temp/${pjManifest.name}.css`), css, err => {
                                        if (!err) {
                                            console.log(`Created ${chalk.blue(path.join(process.cwd(), `temp/${pjManifest.name}.css`))}`);
                                            callback(null);
                                        }
                                    });
                                });
                            } else {
                                callback(err);
                                console.log(`Error: ${chalk.red(err)}`);
                            }
                        });
                    },
                    themeMinified(callback) {
                        sass.render({
                            file: path.join(process.cwd(), pjManifest.main),
                            outputStyle: "compressed",
                            sourceMap: true,
                            sourceMapContents: true,
                            precision: 6,
                            outFile: path.join(process.cwd(), `temp/${pjManifest.name}.min.css`)
                        }, (err, {css}) => {
                            if (!err) {
                                fs.mkdir(path.join(process.cwd(), "temp"), { recursive: true }, (err) => {
                                    if (err) {
                                        callback(err);
                                        return console.log(`Error: ${chalk.red(err)}`);
                                    }
                                    fs.writeFile(path.join(process.cwd(), `temp/${pjManifest.name}.min.css`), css, err => {
                                        if (!err) {
                                            console.log(`Created ${chalk.blue(path.join(process.cwd(), `temp/${pjManifest.name}.min.css`))}`);
                                            callback(null);
                                        }
                                    });
                                });
                            } else {
                                callback(err);
                                console.log(`Error: ${chalk.red(err)}`);
                            }
                        });
                    },
                    themeAsar(callback) {
                        console.log("\r\nCompressing all files..");
                        fs.mkdir(path.join(process.cwd(), "temp"), { recursive: true }, (err) => {
                            if (err) {
                                callback(err);
                                return console.log(`Error: ${chalk.red(err)}`);
                            }
                            asar.createPackage(path.join(process.cwd(), "temp"), path.join(process.cwd(), `build/${pjManifest.name}.asar`), function() {
                                console.log(`Created ${chalk.blue(path.join(process.cwd(), `build/${pjManifest.name}.asar`))}`);
                                callback(null);
                            });
                        });
                    },
                    removeCss(callback) {
                        console.log("\r\nRemoving temp files..");
                        del([path.join(process.cwd(), "temp")]).then(paths => {
                            console.log(`Deleted ${chalk.blue(paths.join("\r\n"))}`);
                            callback(null);
                        });
                    }
                }, err => {
                    if (!err) return console.log(chalk.green("Done!"));
                    console.log("Done, but there was some errors. See above.");
                });
            }
            // Build the extension..
            else {
                console.log("Compressing all files..");
                fs.mkdir(path.join(process.cwd(), "build"), { recursive: true }, (err) => {
                    if (err) {
                        return console.log(`Error: ${chalk.red(err)}`);
                    }
                    asar.createPackageWithOptions(path.join(process.cwd()), path.join(process.cwd(), `build/${pjManifest.name}.asar`), { globOptions: { cwd: process.cwd(), ignore: ["{build}/**", "**/*.asar"] } }, function() {
                        console.log(`Created ${chalk.blue(path.join(process.cwd(), `build/${pjManifest.name}.asar`))}`);
                    });
                });
            }
        }
        else {
            console.log(`\r\n${chalk.red("Error!")} Couldn't locate the "package.json" file. Please use this command inside your project's folder.`);
        }
    });

// Generate a new theme from templates..
function createTemplateContents(templatePath, newProjectPath) {
    const filesToCreate = fs.readdirSync(templatePath);

    // For each file found in templates directory..
    filesToCreate.forEach(file => {
        const origFilePath = `${templatePath}/${file}`;

        // Get stats about the current file..
        const stats = fs.statSync(origFilePath);

        // Current file is really a file..
        if (stats.isFile()) {
            const contents = fs.readFileSync(origFilePath, "utf8");

            const writePath = `${process.cwd()}/${newProjectPath}/${file}`;
            fs.writeFileSync(writePath, contents, "utf8");
        }

        // Current file is actually a directory..
        else if (stats.isDirectory()) {
            fs.mkdirSync(`${process.cwd()}/${newProjectPath}/${file}`);

            // Recursive call..
            createTemplateContents(`${templatePath}/${file}`, `${newProjectPath}/${file}`);
        }
    });
}

// Generate a package.json file..
function createManifest(config) {
    // Pretty-print JSON..
    let data = JSON.stringify(Object.assign({}, {
        version: "0.0.1"
    }, config), null, 2);

    // Write file to disk..
    const writePath = `${process.cwd()}/${config.name}/package.json`;
    fs.writeFileSync(writePath, data, "utf8");
}

program.parse(process.argv);

// Catch unhandledRejection..
process.on("unhandledRejection", () => {
    //
});
